@@ -24,6 +24,9 @@
 #include <QGraphicsScene>
 #include <QGraphicsRectItem>
 #include <QTime>
+#include <QGraphicsRotation>
+#include <QPropertyAnimation>
+#include <QVector3D>
 
 #include <cmath>
 
